{% for entry in tree %}

## {{ entry.version }}{% if entry.date %} ({{ entry.date }}){% endif %}

{% for change_key, changes in entry.changes|dictsort(reverse=True) %}

{% if change_key %}
### {{ change_key|trim }}
{% endif %}

{% for change in changes %}
{% if change.issue %}
- {{change.scope}} [`{{ change.sha1 | truncate(7, end="") }}`](https://github.com/JenspederM/kedro-databricks/commit/{{ change.sha1 }}): [{{ change.message }}](https://github.com/JenspederM/kedro-databricks/pull/{{ change.issue }})
{% else %}
- {{change.scope}} [`{{ change.sha1 | truncate(7, end="") }}`](https://github.com/JenspederM/kedro-databricks/commit/{{ change.sha1 }}): {{ change.message }}
{% endif %}
{% endfor %}
{% endfor %}
{% endfor %}
